/*******************************************************
 * uOLED-160-GMD1 Driver for Arduino
 * 
 * Updated by Mustafa Bagdatli
 * http://mustafabagdatli.com
 * 12/02/2010
 * 
 * Original code by Oscar Gonzalez
 * www.bricogeek.com
 * August 2007
 * 
 * 
 * 
 * 
 *******************************************************/



#define OLED_BAUDRATE                                           9600//57600

#define OLED_RESETPIN                                           8      // PIN of reset

#define OLED_INITDELAYMS                                        5000



// Initialisation routine

#define	OLED_DETECT_BAUDRATE					0x55



// Drawing routines

#define	OLED_CLEAR						0x45

#define	OLED_BKGCOLOR						0x42

#define	OLED_COPYPASTE						0x63



#define	OLED_LINE						0x4C

#define	OLED_CIRCLE						0x43

#define	OLED_CIRCLEFILL						0x69

#define	OLED_PUTPIXEL						0x50

#define	OLED_READPIXEL						0x52

#define	OLED_RECTANGLE						0x72

#define	OLED_PAINTAREA						0x70



// Text properties

#define	OLED_SETFONTSIZE					0x46

#define	OLED_FONT5X7					0x01

#define	OLED_FONT8X8					0x02

#define	OLED_FONT8X12					0x03

#define	OLED_TEXTFORMATED					0x54

#define OLED_TEXTOPAQUE                                  0x4F


// OLED Control

#define	OLED_COMMAND_CONTROL					0x59

#define	OLED_COMMAND_DISPLAY					0x01

#define	OLED_COMMAND_CONTRAST					0x02

#define	OLED_COMMAND_POWER					0x03



#define OLED_ACK  0x06  // Ok

#define OLED_NAK  0x15  // Error

#define OLED_IMAGE                                          0x49
#define OLED_USD                                            0x40
#define OLED_SECTOR1                                        0x10
#define OLED_SECTOR2                                        0x000000


void OLED_ResetDisplay()

{

  digitalWrite(OLED_RESETPIN, LOW);

  delay(20);                  

  digitalWrite(OLED_RESETPIN, HIGH);

  delay(20);  



}



char OLED_GetResponse()

{

  byte incomingByte = OLED_ACK;



  // Wait for data avaliable

  while (!Serial1.available()) { 
    delay(1); 
  }



  // Read incoming byte

  incomingByte = Serial1.read();



  return incomingByte;

}



// Initialise OLED display. You must first activate a Serial1 comunication!

void OLED_Init()

{

  // First reset display

  OLED_ResetDisplay();  



  delay(OLED_INITDELAYMS);                       // Wait for init

  // Autodetect baudrate

  Serial1.print(OLED_DETECT_BAUDRATE, BYTE); 

  OLED_GetResponse(); 



}



// Get 16bits value from RGB (0 to 63, 565 format)

int GetRGB(int red, int green, int blue)

{



  char text[255];



  int outR = ((red * 31) / 255);

  int outG = ((green * 63) / 255);

  int outB = ((blue * 31) / 255);



  return (outR << 11) | (outG << 5) | outB;

}



void OLED_Clear()

{



  Serial1.print(OLED_CLEAR, BYTE); // Pixel write

  delay(20);

  OLED_GetResponse();



}



void OLED_PutPixel(char x, char y, int color)

{



  // Putpixel

  Serial1.print(OLED_PUTPIXEL, BYTE); // Pixel write

  Serial1.print(x, BYTE);	// x

  Serial1.print(y, BYTE);



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB



  OLED_GetResponse();



}



void OLED_DrawLine(char x1, char y1, char x2, char y2, int color)

{



  // Line drawing

  Serial1.print(OLED_LINE, BYTE); // Line



  Serial1.print(x1, BYTE);	// x1

  Serial1.print(y1, BYTE);		// y1

  Serial1.print(x2, BYTE);	// x2

  Serial1.print(y2, BYTE);		// y2



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB



  OLED_GetResponse();



}



void OLED_DrawRectangle(char x, char y, char width, char height, char filled, int color)

{



  //int color = 0xFFFF;



  // Rectangle drawing

  Serial1.print(OLED_RECTANGLE, BYTE); 



  Serial1.print(x, BYTE);			// x1

  Serial1.print(y, BYTE);		// y1



  Serial1.print(x+width, BYTE);		// x2

  Serial1.print(y+height, BYTE);		// y1



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB

  /*

   	if (filled == 1) { Serial1.print(0x01); }	// Filled
   
   	else { Serial1.print(0x00); }				// Outline
   
   	*/

  OLED_GetResponse();

}



void OLED_DrawCircle(char x, char y, char radius, char filled, int color)

{

  Serial1.print(OLED_CIRCLE, BYTE); 



  Serial1.print(x, BYTE);	// x

  Serial1.print(y, BYTE);	// y

  Serial1.print(radius, BYTE);	// radius



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB



  OLED_GetResponse();



}



/*

 Change font format:
 
 FontType can be:
 
 OLED_FONT5X7
 
 OLED_FONT8X8
 
 OLED_FONT8X12
 
 */

void OLED_SetFontSize(char FontType)

{

  Serial1.print(OLED_SETFONTSIZE, BYTE);

  Serial1.print(FontType, BYTE);



  OLED_GetResponse();

}



void OLED_DrawText(char column, char row, char font_size, char *mytext, int color)

{

  //char mytext[] = "Arkanoid by pK";



  Serial1.print(0x73, BYTE); // s (Formated text)



  //Serial1.print(column); // column



  // Adjust to center of the screen (26 Columns at font size 0)

  int newCol = 13 - (strlen(mytext)/2);

  //Serial1.print(newCol); // column

  Serial1.print(column, BYTE); // column



  Serial1.print(row, BYTE); // row

  Serial1.print(font_size, BYTE); // font size (0 = 5x7 font, 1 = 8x8 font, 2 = 8x12 font)



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB



  for (int i=0 ; i<strlen(mytext) ; i++)

  {

    Serial1.print(mytext[i], BYTE); // character to write

  }

  Serial1.print(0x00, BYTE); // string terminator (always 0x00)



  OLED_GetResponse();



}



void OLED_DrawSingleChar(char column, char row, char font_size, char MyChar, int color)

{



  Serial1.print(OLED_TEXTFORMATED, BYTE); // T (Formated)



  Serial1.print(MyChar, BYTE);

  Serial1.print(column, BYTE);

  Serial1.print(row, BYTE);



  // Color

  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);	// LSB



  OLED_GetResponse();



}

//========== Added by Eric ============

//==== Set text transparency    (Eric)
void OLED_SetTextTrans(char FontTrans){

  Serial1.print(OLED_TEXTOPAQUE, BYTE);
  Serial1.print(FontTrans, BYTE);

  OLED_GetResponse();
}

//==== Set PenSize    (Eric)
void OLED_PenSize(char PenSize){

  Serial1.print(OLED_PAINTAREA, BYTE);
  Serial1.print(PenSize, BYTE);

  OLED_GetResponse();
}

//==== Show SD image   (Eric)
void OLED_ShowImage(char PosX, char PosY, char Width, char Height, char colourMode, char SectorHi, char SectorMid, char SectorLo){

  
    Serial1.print(OLED_USD, BYTE);
    Serial1.print(OLED_IMAGE, BYTE); 
    Serial1.print(PosX, BYTE);
    Serial1.print(PosY, BYTE);
    Serial1.print(Width, BYTE);
    Serial1.print(Height, BYTE);
    Serial1.print(colourMode, BYTE);
    Serial1.print(SectorHi, BYTE);
    Serial1.print(SectorMid, BYTE);
    Serial1.print(SectorLo, BYTE);


  OLED_GetResponse();
}

//==== Change BackGround color  (Eric)
void OLED_Background(int color){

    Serial1.print(OLED_BKGCOLOR, BYTE);
    
     // Color
  Serial1.print(color >> 8, BYTE);		// MSB			

  Serial1.print(color & 0xFF, BYTE);		// LSB
  
  OLED_GetResponse();
}
